/* vim: set shiftwidth=4 tabstop=4 autoindent noexpandtab copyindent: */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the leak-monitor extension.
 *
 * The Initial Developer of the Original Code is the Mozilla Foundation.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   L. David Baron <dbaron@dbaron.org> (original author)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* Dialog that comes up to tell user about leaks. */

const CI = Components.interfaces;

var gTreeView = new XULTreeView();

function LeakAlertOnLoad()
{
	document.getElementById("lwjs-tree").view = gTreeView;

	var lwjss = window.arguments[0].QueryInterface(CI.leakmonIReport)
		.getLeakedWrappedJSs({});

	for each (var lwjs in lwjss) {
		gTreeView.childData.appendChild(new JSObjectRecord(lwjs));
	}

	JSConsole.init();
	JSConsole.dumpReport(lwjss);
}

function LeakAlertOnUnload()
{
}

function JSObjectRecord(lwjs)
{
	this.lwjs = lwjs;

	this.setColumnPropertyName("name", "name");
	this.setColumnPropertyName("filename", "fileName");
	this.setColumnPropertyName("linestart", "lineStart");
	this.setColumnPropertyName("lineend", "lineEnd");
	this.setColumnPropertyName("string", "stringRep");
	for each (var m in
	          ["name", "fileName", "lineStart", "lineEnd", "stringRep"]) {
		this[m] = lwjs[m];
	}
	if (!this.fileName) {
		this.lineStart = "";
		this.lineEnd = "";
	}

	if (lwjs.numProperties > 0)
		this.reserveChildren(true);
}

JSObjectRecord.prototype = new XULTreeViewRecord(gTreeView.share);

JSObjectRecord.prototype.onPreOpen = function() {
	this.childData = new Array();
	for (var i = 0; i < this.lwjs.numProperties; ++i) {
		this.appendChild(new JSObjectRecord(this.lwjs.getPropertyAt(i)));
	}
}

var JSConsole = {

/*
 * From JS Console output redirector by TataBlack
 * http://tblack.f2o.org/
 * Licence unknown but assumed to be MPL
*/

  PREF:      Components.classes["@mozilla.org/preferences-service;1"]
             .getService(CI.nsIPrefService),
  CONSOLE:   Components.classes["@mozilla.org/consoleservice;1"]
             .getService(CI.nsIConsoleService),
  OUTPUTDIR: Components.classes["@mozilla.org/file/directory_service;1"]
             .getService(CI.nsIProperties)
             .get("ProfD", CI.nsIFile),
  LOGFILE:   "leak.log",
  CONVERTER: Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
             .createInstance(CI.nsIScriptableUnicodeConverter),

  gDateService: Components.classes["@mozilla.org/intl/scriptabledateformat;1"]
                .getService(CI.nsIScriptableDateFormat),

  init: function jsc_init () {
    this.CONVERTER.charset = "UTF-8";
    this.OUTPUTDIR.append("JSConsoleOutput");

    if(!this.OUTPUTDIR.exists()) {
      this.OUTPUTDIR.create(CI.nsIFile.DIRECTORY_TYPE, 0664);
    }

    /* Append the log file name */
    this.OUTPUTDIR.append(this.LOGFILE);				

    /* Check again, now for log file existence */
    if(!this.OUTPUTDIR.exists()) {
      this.OUTPUTDIR.create(CI.nsIFile.NORMAL_FILE_TYPE, 0664);
    }

  },

  writeToFile: function jsc_fileWrite (aMessage) {
    try {
      aContent = this.CONVERTER.ConvertFromUnicode(aMessage);

      var ostream = Components.classes['@mozilla.org/network/file-output-stream;1']
                    .createInstance(CI.nsIFileOutputStream);
      ostream.init(this.OUTPUTDIR, 0x02 | 0x08 | 0x10, 0x200, false);
      ostream.write(aContent, aContent.length);
      ostream.close();
    } catch(ex) {
      alert("Unable to redirect console output to a file.");
    }
  },

// End of JS Console output redirector code

// 2006-05-25: Philip Chee: Dump output to %ProfD%/JSConsoleOutput/leak.log

  dumpReport: function jsc_dumpRep (lwjss) {
    var now = new Date();
    var sDate = this.gDateService.FormatDateTime(
      "",
      this.gDateService.dateFormatLong,
      this.gDateService.timeFormatSeconds, 
      now.getFullYear(),
      now.getMonth()+1,
      now.getDate(),
      now.getHours(),
      now.getMinutes(),
      now.getSeconds());
    this.writeToFile("[" + sDate + "]\n");
    
    for each (var lwjs in lwjss) {
      this.dumpRecord(lwjs, 0);
    }
  },

  dumpRecord: function jsc_dumpRec (lwjs, indent) {
    var aRec = new Array();
    var aTitles = ["name", "fileName", "lineStart", "lineEnd", "stringRep"];

    for each (var m in aTitles) {
      aRec[m] = lwjs[m];
    }
    if (!aRec.fileName) {
      aRec.lineStart = "";
      aRec.lineEnd = "";
    }

    var margin = "";
    for (var mg =0; mg < indent; mg++) {
      margin = margin + ".";
    }
    var aMessage = "";
    for each (var nt in aTitles) {
      aMessage = aMessage + margin + nt + ": " + aRec[nt] + "\n"; 
    }
    this.writeToFile(aMessage);

    for (var i = 0; i < lwjs.numProperties; ++i) {
      this.dumpRecord(lwjs.getPropertyAt(i), indent + 1);
	}
	if (indent < 1) {
      this.writeToFile("\n");
    }
  }
}